@@ -4,12 +4,15 @@ use crate::radius::{Radius, RadiusOption};
 use boolector::BV;
 use clap::{App, Arg};
 use std::time::Instant;
+use std::fs;
+use std::path::Path;
 
-// use crate::state::State;
+use crate::state::StateStatus;
 use crate::value::Value;
 
 use ahash::AHashMap;
 type HashMap<P, Q> = AHashMap<P, Q>;
+use std::collections::VecDeque;
 
 pub mod memory;
 pub mod operations;
@@ -97,10 +100,17 @@ fn main() {
                 .help("Do not simulate imports"),
         )
         .arg(
-            Arg::with_name("frida")
+            Arg::with_name("fuzz")
                 .short("F")
-                .long("frida")
-                .help("Create initial state from frida hook"),
+                .long("fuzz")
+                .takes_value(true)
+                .help("Generate testcases and write to supplied dir"),
+        )
+        .arg(
+            Arg::with_name("max")
+                .long("max")
+                .takes_value(true)
+                .help("Maximum number of states to keep at a time"),
         )
         .arg(
             Arg::with_name("profile")
@@ -108,6 +118,12 @@ fn main() {
                 .long("profile")
                 .help("Get performance and runtime information"),
         )
+        .arg(
+            Arg::with_name("color")
+                .short("C")
+                .long("color")
+                .help("Use color output"),
+        )
         .arg(
             Arg::with_name("stdin")
                 .short("0")
@@ -256,8 +272,10 @@ fn main() {
 
     let libpaths: Vec<&str> = collect!(matches, "libs");
 
+    let debug = occurs!(matches, "verbose") || occurs!(matches, "color");
     let no_sims = occurs!(matches, "no_sims");
     let profile = occurs!(matches, "profile");
+    let fuzz    = occurs!(matches, "fuzz");
     let all_sims = !no_sims && libpaths.is_empty();
 
     let plugins = occurs!(matches, "plugins")
@@ -267,10 +285,11 @@ fn main() {
             .starts_with("frida:"); // load plugins for r2frida
 
     let mut options = vec![
-        RadiusOption::Debug(occurs!(matches, "verbose")),
+        RadiusOption::Debug(debug),
         RadiusOption::Lazy(occurs!(matches, "lazy")),
         RadiusOption::Strict(occurs!(matches, "strict")),
         RadiusOption::SelfModify(occurs!(matches, "selfmodify")),
+        RadiusOption::ColorOutput(occurs!(matches, "color")),
         RadiusOption::Sims(!no_sims),
         RadiusOption::SimAll(all_sims),
         RadiusOption::LoadLibs(!libpaths.is_empty()),
@@ -289,8 +308,20 @@ fn main() {
 
     let start = Instant::now();
 
+    let path = matches.value_of("path").unwrap_or_default();
+    let dir = Path::new(matches.value_of("fuzz").unwrap_or("."));
     let mut radius = Radius::new_with_options(matches.value_of("path"), &options);
 
+    if !dir.exists() {
+        fs::create_dir(&dir).unwrap();
+    }
+
+    let max_states = matches
+        .value_of("max")
+        .unwrap_or("256")
+        .parse()
+        .unwrap_or(256);
+
     // execute provided r2 commands
     let cmds: Vec<&str> = collect!(matches, "r2_command");
     for cmd in cmds {
@@ -302,8 +333,10 @@ fn main() {
 
     let mut state = if let Some(address) = matches.value_of("address") {
         let addr = radius.get_address(address).unwrap();
-        if occurs!(matches, "frida") {
+        if path.starts_with("frida:") {
             radius.frida_state(addr)
+        } else if path.starts_with("gdb:") || path.starts_with("dbg:") {
+            radius.debug_state(addr)
         } else {
             radius.call_state(addr)
         }
@@ -341,7 +374,14 @@ fn main() {
                 let value = if let Some(sym) = symbol_map.get(arg) {
                     Value::Symbolic(sym.clone(), 0)
                 } else {
-                    let bytes: Vec<Value> = arg
+                    // @ signs to prevent parsing as radius args
+                    let narg = if arg.starts_with("@") {
+                        &arg[1..]
+                    } else {
+                        &arg[..]
+                    };
+
+                    let bytes: Vec<Value> = narg
                         .as_bytes()
                         .iter()
                         .map(|b| Value::Concrete(*b as u64, 0))
@@ -454,10 +494,12 @@ fn main() {
         .map(|x| radius.get_address(x).unwrap())
         .collect();
 
+    let mut analyzed = false;
     // get code references to strings and add them to the avoid list
     if occurs!(matches, "avoid_strings") {
         // need to analyze to get string refs
         radius.analyze(3);
+        analyzed = true;
         for string in collect!(matches, "avoid_strings") {
             for location in radius.r2api.search_strings(string).unwrap() {
                 avoid.extend(
@@ -475,14 +517,14 @@ fn main() {
     // get code references to strings and add them to the breakpoints
     if occurs!(matches, "break_strings") {
         // need to analyze to get string refs
-        radius.analyze(3);
+        if !analyzed { radius.analyze(3); }
         for string in collect!(matches, "break_strings") {
             for location in radius.r2api.search_strings(string).unwrap() {
                 bps.extend(
                     radius
                         .r2api
                         .get_references(location)
-                        .unwrap()
+                        .unwrap_or_default()
                         .iter()
                         .map(|x| x.from),
                 );
@@ -507,66 +549,138 @@ fn main() {
     }
 
     if profile {
-        println!("init time: {}", start.elapsed().as_micros());
+        println!("init time:\t{}", start.elapsed().as_micros());
     }
     // run the thing
     let run_start = Instant::now();
 
-    let result = radius.run(state, threads);
+    if !fuzz {
+        let result = radius.run(state, threads);
 
-    if profile {
-        let usecs = run_start.elapsed().as_micros();
-        let steps = radius.get_steps();
-        println!(
-            "run time: {} ins: {} ins/usec: {}",
-            usecs,
-            steps,
-            (steps as f64 / usecs as f64)
-        );
-    }
-
-    if let Some(mut end_state) = result {
-        // collect the ESIL strings to evaluate after running
-        let evals: Vec<&str> = collect!(matches, "evaluate_after");
-
-        for eval in evals {
-            radius.processor.parse_expression(&mut end_state, eval);
+        if profile {
+            let usecs = run_start.elapsed().as_micros();
+            let steps = radius.get_steps();
+            println!(
+                "run time:\t{}\ninstructions:\t{}\ninstr/usec:\t{:0.6}",
+                usecs,
+                steps,
+                (steps as f64 / usecs as f64)
+            );
         }
 
-        let solve_start = Instant::now();
+        if let Some(mut end_state) = result {
+            // collect the ESIL strings to evaluate after running
+            let evals: Vec<&str> = collect!(matches, "evaluate_after");
 
-        for symbol in symbol_map.keys() {
-            let val = Value::Symbolic(end_state.translate(&symbol_map[symbol]).unwrap(), 0);
+            for eval in evals {
+                radius.processor.parse_expression(&mut end_state, eval);
+            }
 
-            if let Some(bv) = end_state.solver.eval_to_bv(&val) {
-                let str_opt = end_state.evaluate_string(&bv);
-                let sym_type = symbol_types[symbol];
-                if sym_type == "str" && str_opt.is_some() {
-                    println!("  {} : {:?}", symbol, str_opt.unwrap());
+            let solve_start = Instant::now();
+
+            for symbol in symbol_map.keys() {
+                let val = Value::Symbolic(end_state.translate(&symbol_map[symbol]).unwrap(), 0);
+
+                if let Some(bv) = end_state.solver.eval_to_bv(&val) {
+                    let str_opt = end_state.evaluate_string(&bv);
+                    let sym_type = symbol_types[symbol];
+                    if sym_type == "str" && str_opt.is_some() {
+                        println!("  {} : {:?}", symbol, str_opt.unwrap());
+                    } else {
+                        let hex = &format!("{:?}", bv)[2..];
+                        println!("  {} : 0x{}", symbol, hex);
+                    }
                 } else {
-                    let hex = &format!("{:?}", bv)[2..];
-                    println!("  {} : 0x{}", symbol, hex);
+                    println!("  {} : no satisfiable value", symbol);
+                }
+            }
+
+            if profile {
+                println!("solve time:\t{}", solve_start.elapsed().as_micros());
+            }
+
+            // dump program output
+            if occurs!(matches, "stdout") {
+                print!("{}", end_state.dump_file_string(1).unwrap_or_default());
+            }
+            if occurs!(matches, "stderr") {
+                print!("{}", end_state.dump_file_string(2).unwrap_or_default());
+            }
+        }
+    } else {
+        let mut pcs: HashMap<u64, usize> = HashMap::new();
+        let mut states = VecDeque::new();
+        let mut solutions: HashMap<Vec<u8>, usize> = HashMap::new();
+        states.push_back(state);
+
+        let mut file_counts: HashMap<&str, usize> = HashMap::new();
+            
+        for symbol in symbol_map.keys() {
+            file_counts.insert(symbol, 0);
+        }
+
+        while !states.is_empty() {
+            let num_states = states.len();
+            let mut s = states.pop_front().unwrap();
+            let cpc = s.registers.get_pc().as_u64().unwrap();
+
+            radius.processor.fetch_instruction(&mut s, cpc);
+            let tn = radius.processor.instructions[&cpc].instruction.type_num;
+
+            let mut new_states = radius.processor.step(&mut s);
+            new_states.push(s);
+
+            for mut new_state in new_states {
+                let pc = new_state.registers.get_pc().as_u64().unwrap();
+                let active = new_state.status == StateStatus::Active;
+                if pcs.contains_key(&pc) {
+                    let count = pcs[&pc];
+                    pcs.insert(pc, count + 1);
+                    if active && num_states <= max_states {
+                        states.push_back(new_state);
+                    }
+                    continue;
+                }
+                pcs.insert(pc, 1);
+                // after (conditional) calls and jumps
+                if tn & 0xf == 1 || tn & 0xf == 4 {
+                    for symbol in symbol_map.keys() {
+                        let val = new_state
+                            .translate(&symbol_map[symbol])
+                            .unwrap();
+                        
+                        if let Some(bytes) = new_state.evaluate_bytes(&val) {
+                            if !solutions.contains_key(&bytes) {
+                                let c = file_counts[symbol];
+                                let filename = &format!("{}{:04}", symbol, c);
+                                fs::write(dir.join(filename), &bytes).unwrap();
+                                file_counts.insert(symbol, c + 1);
+                                solutions.insert(bytes, 1);
+                            }
+                        }
+                    }
+                }
+                if active {
+                    states.push_front(new_state);
                 }
-            } else {
-                println!("{} : no satisfiable value", symbol);
             }
         }
 
         if profile {
-            println!("solve time: {}", solve_start.elapsed().as_micros());
-        }
-
-        // dump program output
-        if occurs!(matches, "stdout") {
-            print!("{}", end_state.dump_file_string(1).unwrap_or_default());
-        }
-        if occurs!(matches, "stderr") {
-            print!("{}", end_state.dump_file_string(2).unwrap_or_default());
+            let usecs = run_start.elapsed().as_micros();
+            let steps = radius.get_steps();
+            println!(
+                "run time:\t{}\ninstructions:\t{}\ninstr/usec:\t{:0.6}\ngenerated:\t{}",
+                usecs,
+                steps,
+                (steps as f64 / usecs as f64),
+                file_counts.values().sum::<usize>()
+            );
         }
     }
 
     if profile {
-        println!("total time: {}", start.elapsed().as_micros());
+        println!("total time:\t{}", start.elapsed().as_micros());
     }
 
     radius.close();
