@@ -29,8 +29,10 @@ const PROT_EXEC: u64 = 0x4;
 pub struct Memory {
     pub solver: Solver,
     pub r2api: R2Api,
+    
+    // TODO refactor merge to make this private
     pub mem: HashMap<u64, Value>,
-    pub heap: Heap,
+    heap: Heap,
     pub bits: u64,
     pub endian: Endian,
     pub segs: Vec<MemorySegment>,
@@ -99,7 +101,7 @@ impl Memory {
     }
 
     pub fn free(&mut self, addr: &Value) -> Value {
-        let address = addr.as_u64().unwrap(); //solver.evalcon_to_u64(addr).unwrap();
+        let address = addr.as_u64().unwrap(); 
         if let Some(ret) = self.heap.free(address) {
             Value::Concrete(ret, addr.get_taint())
         } else {
@@ -502,8 +504,6 @@ impl Memory {
                 }
             }
         }
-        //println!("read {:?}", data);
-        //data
     }
 
     pub fn prot_to_str(&self, prot: u64) -> String {
@@ -551,7 +551,7 @@ impl Memory {
 
     pub fn write_string(&mut self, addr: u64, string: &str) {
         let data = string.as_bytes();
-        let mut data_value = vec![];
+        let mut data_value = Vec::with_capacity(string.len());
         for d in data {
             data_value.push(Value::Concrete(*d as u64, 0));
         }
@@ -706,14 +706,7 @@ impl Heap {
             self.chunks.pop();
             Some(addr)
         } else {
-            let mut rem = 0;
-            for (i, chunk) in self.chunks.iter().enumerate() {
-                if chunk.addr == addr {
-                    rem = i;
-                    break;
-                }
-            }
-            if rem != 0 {
+            if let Some(rem) = self.chunks.iter().position(|x| x.addr == addr) {
                 self.chunks.remove(rem);
                 Some(addr)
             } else {
