@@ -17,7 +17,7 @@ use std::mem;
 type HashMap<P, Q> = AHashMap<P, Q>;
 
 const INSTR_NUM: usize = 64;
-const COLOR: bool = false;
+// const COLOR: bool = true;
 const CALL_TYPE: i64 = 3;
 const RETN_TYPE: i64 = 5;
 // const NOP_TYPE: i64 = 8;
@@ -43,13 +43,14 @@ pub struct Processor {
     pub breakpoints: AHashSet<u64>,
     pub mergepoints: AHashSet<u64>,
     pub avoidpoints: AHashSet<u64>,
-    //pub visited: AHashSet<u64>,
+    pub visited: AHashSet<u64>,
     pub merges: HashMap<u64, State>,
     pub selfmodify: bool,
     pub optimized: bool,
     pub debug: bool,
     pub lazy: bool,
     pub force: bool,
+    pub color: bool,
     pub topological: bool, // execute blocks in topological sort order
     pub steps: u64,        // number of state steps
 }
@@ -67,9 +68,9 @@ pub enum InstructionStatus {
 
 #[derive(Debug, Clone)]
 pub struct InstructionEntry {
-    instruction: Instruction,
-    tokens: Vec<Word>,
-    status: InstructionStatus, // next: Option<Arc<InstructionEntry>>
+    pub instruction: Instruction,
+    pub tokens: Vec<Word>,
+    pub status: InstructionStatus, // next: Option<Arc<InstructionEntry>>
 }
 
 //const DEBUG: bool = false; // show instructions
@@ -87,6 +88,7 @@ impl Processor {
         lazy: bool,
         force: bool,
         topological: bool,
+        color: bool
     ) -> Self {
         Processor {
             instructions: HashMap::new(),
@@ -98,7 +100,7 @@ impl Processor {
             breakpoints: AHashSet::new(),
             mergepoints: AHashSet::new(),
             avoidpoints: AHashSet::new(),
-            //visited:      AHashSet::new(),
+            visited: AHashSet::new(),
             merges: HashMap::new(),
             selfmodify,
             optimized,
@@ -106,6 +108,7 @@ impl Processor {
             lazy,
             force,
             topological,
+            color,
             steps: 0, //states: vec!()
         }
     }
@@ -126,18 +129,18 @@ impl Processor {
                 tokens.push(operator);
 
             // all this garbage is for the combo ones like ++=[8] ...
-            } else if l > 1 && &s[l - 1..] == "=" && OPS.contains(&&s[0..l - 1]) {
+            } else if l > 1 && &s[l - 1..] == "=" && OPS.contains(&&s[..l - 1]) {
                 let reg_word = tokens.pop().unwrap();
                 tokens.push(reg_word.to_owned());
-                let operator = self.get_operator(&s[0..l - 1]).unwrap();
+                let operator = self.get_operator(&s[..l - 1]).unwrap();
                 tokens.push(operator);
                 tokens.push(reg_word);
                 tokens.push(Word::Operator(Operations::Equal));
-            } else if l > 4 && &s[l - 1..] == "]" && OPS.contains(&&s[0..l - 4]) {
+            } else if l > 4 && &s[l - 1..] == "]" && OPS.contains(&&s[..l - 4]) {
                 tokens.push(Word::Operator(Operations::AddressStore));
                 let peek = self.get_operator(&s[l - 3..]).unwrap();
                 tokens.push(peek);
-                let operator = self.get_operator(&s[0..l - 4]).unwrap();
+                let operator = self.get_operator(&s[..l - 4]).unwrap();
                 tokens.push(operator);
                 let poke = self.get_operator(&s[l - 4..]).unwrap();
                 tokens.push(Word::Operator(Operations::AddressRestore));
@@ -193,7 +196,7 @@ impl Processor {
 
     /// print instruction if debug output is enabled
     pub fn print_instr(&self, state: &mut State, instr: &Instruction) {
-        if !COLOR {
+        if !self.color {
             println!(
                 "{:016x}:  {:<40} |  {}",
                 instr.offset, instr.disasm, instr.esil
@@ -539,10 +542,12 @@ impl Processor {
             RETN_TYPE => {
                 if state.backtrace.is_empty() && new_status == &InstructionStatus::None {
                     // try to avoid returning outside valid context
-                    if !self.breakpoints.is_empty() {
+                    if !self.breakpoints.is_empty() 
+                        || !self.esil_hooks.is_empty() {
+
                         new_status = &InstructionStatus::Avoid;
                     } else {
-                        // break if there are no other breakpoints
+                        // break if there are no other breakpoints/hooks
                         new_status = &InstructionStatus::Break;
                     }
                 } else {
@@ -735,7 +740,7 @@ impl Processor {
             if DO_EVENT_HOOKS && state.has_event_hooks {
                 state.do_hooked(
                     &Event::SymbolicExec(EventTrigger::Before),
-                    &EventContext::ExecContext(new_pc.clone()),
+                    &EventContext::ExecContext(new_pc.clone(), vec![]),
                 );
             }
 
@@ -755,7 +760,10 @@ impl Processor {
             if DO_EVENT_HOOKS && state.has_event_hooks {
                 state.do_hooked(
                     &Event::SymbolicExec(EventTrigger::After),
-                    &EventContext::ExecContext(new_pc.clone()),
+                    &EventContext::ExecContext(
+                        new_pc.clone(), 
+                        state.esil.pcs.clone()
+                    ),
                 );
             }
         }
@@ -812,11 +820,25 @@ impl Processor {
             match current_state.status {
                 StateStatus::Active | StateStatus::PostMerge => {
                     let new_states = self.step(current_state);
+
+                    let pc = current_state
+                        .registers
+                        .get_pc()
+                        .as_u64()
+                        .unwrap_or_default();
+
                     for s in new_states {
                         states.push_front(s);
                         index += 1;
                     }
-                    index += 1;
+                    
+                    // if the current address has not been
+                    // seen before, keep executing it 
+                    if self.visited.contains(&pc) {
+                        index += 1;
+                    } else {
+                        self.visited.insert(pc);
+                    }
                 }
                 StateStatus::Merge => {
                     self.merge(states.remove(index).unwrap());
