@@ -70,7 +70,7 @@ pub struct Instruction {
     #[serde(default = "invalid")]
     pub disasm: String,
 
-    #[serde(default = "blank")]
+    #[serde(default)]
     pub esil: String,
 
     pub bytes: String,
@@ -78,13 +78,13 @@ pub struct Instruction {
     #[serde(default = "invalid")]
     pub r#type: String,
 
-    #[serde(default = "zero")]
+    #[serde(default)]
     pub type_num: i64,
 
-    #[serde(default = "zero")]
+    #[serde(default)]
     pub jump: i64,
 
-    #[serde(default = "zero")]
+    #[serde(default)]
     pub fail: i64,
 }
 
@@ -144,7 +144,7 @@ pub struct RegisterInformation {
 pub struct CoreInfo {
     pub file: String,
 
-    #[serde(default = "zero")]
+    #[serde(default)]
     pub size: i64,
     pub mode: String,
     pub format: String,
@@ -479,6 +479,11 @@ impl R2Api {
         r2_result(serde_json::from_str(json.as_str()))
     }
 
+    pub fn get_shellcode(&mut self, cmd: &str) -> R2Result<Vec<u8>> {
+        let result = self.cmd(&format!("gr;gi exec;gc cmd={};g", cmd))?;
+        Ok(hex_decode(&result))
+    }
+
     /*
         arch/ABI      arg1  arg2  arg3  arg4  arg5  arg6  arg7  Notes
     ──────────────────────────────────────────────────────────────
@@ -595,6 +600,17 @@ impl R2Api {
                 ],
                 ret: "r3".to_string(), // TODO errors are in r0
             }),
+            ("xtensa", _) => Ok(CallingConvention {
+                args: vec![
+                    "a6".to_string(),
+                    "a3".to_string(),
+                    "a4".to_string(),
+                    "a5".to_string(),
+                    "a8".to_string(),
+                    "a9".to_string(),
+                ],
+                ret: "a2".to_string(),
+            }),
             _ => Err("calling convention not found".to_owned()),
         }
     }
@@ -649,7 +665,7 @@ impl R2Api {
     }
 
     pub fn search_bytes(&mut self, data: &[u8]) -> R2Result<Vec<SearchResult>> {
-        let json = self.cmd(&format!("/x {}", hex_encode(data)))?;
+        let json = self.cmd(&format!("/xj {}", hex_encode(data)))?;
         r2_result(serde_json::from_str(json.as_str()))
     }
 
@@ -677,9 +693,7 @@ impl R2Api {
     }
 
     pub fn get_register_value(&mut self, reg: &str) -> R2Result<u64> {
-        let cmd = format!("aer {}", reg);
-        let val = self.cmd(cmd.as_str())?;
-        // println!("val: {}", val);
+        let val = self.cmd(&format!("aer {}", reg))?;
         Ok(u64::from_str_radix(&val[2..val.len() - 1], 16).unwrap())
     }
 
@@ -725,8 +739,12 @@ impl R2Api {
         }
     }
 
+    pub fn init_debug(&mut self, addr: u64) {
+        self.cmd(&format!("db {};dc", addr)).unwrap();
+    }
+
     pub fn init_vm(&mut self) {
-        let _r = self.cmd(format!("aei; aeim {} {}", STACK_START, STACK_SIZE).as_str());
+        let _r = self.cmd(&format!("aei; aeim {} {}", STACK_START, STACK_SIZE));
     }
 
     pub fn init_entry(&mut self, args: &[String], vars: &[String]) {
@@ -735,7 +753,7 @@ impl R2Api {
         let env = vars.join(" ");
         self.init_vm();
         // this is very weird but this is how it works
-        let _r = self.cmd(format!(".aeis {} {} {} @ SP", argc, argv, env).as_str());
+        let _r = self.cmd(&format!(".aeis {} {} {} @ SP", argc, argv, env));
     }
 
     pub fn init_frida(&mut self, addr: u64) -> R2Result<HashMap<String, String>> {
@@ -890,7 +908,12 @@ impl R2Api {
 
         for lib in &libs {
             for path in &paths {
-                let lib_path = path.to_owned() + lib;
+                let lib_path = Path::new(path)
+                    .join(lib)
+                    .to_str()
+                    .unwrap_or_default()
+                    .to_owned();
+                    
                 let loaded = full_paths.iter().any(|x| x == &lib_path);
                 if !loaded && Path::new(&lib_path).exists() {
                     let load_addr = (high_addr & 0xfffffffffffff000) + 0x3000; // idk
