@@ -45,6 +45,8 @@ pub enum RadiusOption {
     LoadPlugins(bool),
     /// Load libraries
     LoadLibs(bool),
+    /// use color output from r2
+    ColorOutput(bool),
     /// Path to load library from
     LibPath(String),
 }
@@ -102,7 +104,7 @@ impl Radius {
      * ```
      */
     pub fn new_with_options<T: AsRef<str>>(filename: Option<T>, options: &[RadiusOption]) -> Self {
-        let mut argv = vec![];
+        let mut argv = vec!["-2"];
         let mut eval_max = 256;
         let mut paths = vec![];
         for o in options {
@@ -118,29 +120,28 @@ impl Radius {
         let debug = options.contains(&RadiusOption::Debug(true));
         let use_sims = !options.contains(&RadiusOption::Sims(false));
 
+        if !options.contains(&RadiusOption::LoadPlugins(true)) {
+            argv.push("-NN");
+        }
+
         if debug {
             // pretty print disasm + esil
             argv.push("-e scr.color=3");
             argv.push("-e asm.cmt.esil=true");
             argv.push("-e asm.lines=false");
             argv.push("-e asm.emu=false");
-        }
-
-        argv.push("-2");
-
-        if !options.contains(&RadiusOption::LoadPlugins(true)) {
-            argv.push("-NN");
-        }
-
-        // need this for sims
-        if use_sims {
-            argv.push("-e io.cache=true");
-            argv.push("-e bin.cache=true");
+            argv.push("-e asm.xrefs=false");
+            argv.push("-e asm.functions=false");
         }
 
         let args = if !argv.is_empty() { Some(argv) } else { None };
 
         let mut r2api = R2Api::new(filename, args);
+        r2api.set_option("io.cache", "true").unwrap();
+        r2api.cmd("eco darkda").unwrap(); // i like darkda
+
+        // doesnt work with io.cache? and not needed?
+        // r2api.set_option("bin.cache", "true").unwrap();
 
         let arch = &r2api.info.bin.arch;
 
@@ -151,13 +152,14 @@ impl Radius {
 
         let lazy = !options.contains(&RadiusOption::Lazy(false));
         let force = options.contains(&RadiusOption::Force(true));
-        let topological = options.contains(&RadiusOption::Topological(true));
+        let topo = options.contains(&RadiusOption::Topological(true));
         let check = options.contains(&RadiusOption::Permissions(true));
         let sim_all = options.contains(&RadiusOption::SimAll(true));
         let selfmod = options.contains(&RadiusOption::SelfModify(true));
         let strict = options.contains(&RadiusOption::Strict(true));
+        let color = options.contains(&RadiusOption::ColorOutput(true));
 
-        let mut processor = Processor::new(selfmod, opt, debug, lazy, force, topological);
+        let mut processor = Processor::new(selfmod, opt, debug, lazy, force, topo, color);
         let processors = Arc::new(Mutex::new(vec![]));
 
         if !options.contains(&RadiusOption::Syscalls(false)) {
@@ -214,6 +216,16 @@ impl Radius {
         state
     }
 
+    /// Initialize state from a debugger breakpoint
+    /// the program will block until bp is hit
+    pub fn debug_state(&mut self, addr: u64) -> State {
+        self.r2api.seek(addr);
+        self.r2api.init_debug(addr);
+        self.init_state()
+    }
+
+    /// Initialize state from a frida hook
+    /// the program will block until the hook is hit
     pub fn frida_state(&mut self, addr: u64) -> State {
         self.r2api.seek(addr);
         let mut state = self.init_state();
@@ -291,7 +303,6 @@ impl Radius {
                 );
 
                 state.memory_write_value(&current, &Value::Concrete(addr, 0), ptrlen);
-
                 current = current + Value::Concrete(ptrlen as u64, 0);
             }
             state.memory_write_value(&current, &Value::Concrete(0, 0), ptrlen);
@@ -454,11 +465,12 @@ impl Radius {
         }
 
         let mut handles = Vec::with_capacity(threads);
-        let statevector = Arc::new(Mutex::new(VecDeque::with_capacity(self.eval_max)));
+        let statevector = Arc::new(Mutex::new(VecDeque::with_capacity(threads*self.eval_max)));
         statevector.lock().unwrap().push_back(state);
 
         loop {
             let mut count = 0;
+            let state_count = statevector.lock().unwrap().len();
             while count < threads && !statevector.lock().unwrap().is_empty() {
                 //println!("on thread {}!", thread_count);
                 let procs = self.processors.clone();
@@ -472,7 +484,7 @@ impl Radius {
                 };
 
                 let handle = thread::spawn(move || {
-                    let new_states = processor.run(state, true);
+                    let new_states = processor.run(state, state_count < threads);
                     states.lock().unwrap().extend(new_states);
                     procs.lock().unwrap().push(processor);
                 });
