@@ -2,15 +2,15 @@
 use ahash::AHashMap;
 type HashMap<P, Q> = AHashMap<P, Q>;
 
-use crate::r2_api::{RegisterInfo, AliasInfo, R2Api};
-use crate::value::Value;
+use crate::r2_api::{AliasInfo, R2Api, RegisterInfo};
 use crate::solver::Solver;
+use crate::value::Value;
 
 #[derive(Debug, Clone, PartialEq, Eq, PartialOrd, Ord, Hash)]
 pub struct Bounds {
     start: u64,
     end: u64,
-    size: u64
+    size: u64,
 }
 
 #[derive(Debug, Clone)]
@@ -23,39 +23,40 @@ pub struct Register {
 
 #[derive(Clone)]
 pub struct Registers {
-    pub solver:  Solver,
-    pub r2api:   R2Api,
+    pub solver: Solver,
+    pub r2api: R2Api,
     pub aliases: HashMap<String, AliasInfo>,
-    pub regs:    HashMap<String, Register>,
+    pub regs: HashMap<String, Register>,
     pub indexes: Vec<Register>,
-    pub values:  Vec<Value>,
-    pub pc:      Option<Register>
+    pub values: Vec<Value>,
+    pub pc: Option<Register>,
 }
 
 impl Registers {
     pub fn new(r2api: &mut R2Api, btor: Solver, blank: bool) -> Self {
         let mut reg_info = r2api.get_registers().unwrap();
-        reg_info.reg_info.sort_by(|a, b| b.size.partial_cmp(&a.size).unwrap());
+        reg_info
+            .reg_info
+            .sort_by(|a, b| b.size.partial_cmp(&a.size).unwrap());
 
         let mut registers = Registers {
             solver: btor.clone(),
             r2api: r2api.clone(),
             aliases: HashMap::new(),
-            regs:    HashMap::new(),
-            indexes: vec!(),
-            values:  vec!(), 
-            pc: None
+            regs: HashMap::new(),
+            indexes: vec![],
+            values: vec![],
+            pc: None,
         };
 
-    
-        let mut bounds_map: HashMap<Bounds,usize> = HashMap::new();
+        let mut bounds_map: HashMap<Bounds, usize> = HashMap::new();
         for reg in reg_info.reg_info {
             let mut bounds = Bounds {
                 start: reg.offset,
                 end: reg.offset.wrapping_add(reg.size),
-                size: reg.size
+                size: reg.size,
             };
-    
+
             let old_bounds = bounds_map.keys();
             let mut in_bounds = false;
             for bound in old_bounds {
@@ -64,7 +65,7 @@ impl Registers {
                     bounds = bound.clone();
                 }
             }
-    
+
             if !in_bounds {
                 let val = if !blank {
                     Value::Concrete(r2api.get_register_value(&reg.name).unwrap(), 0)
@@ -76,26 +77,30 @@ impl Registers {
                 bounds_map.insert(bounds.clone(), registers.values.len());
                 registers.values.push(val);
             }
-    
+
             let reg_obj = Register {
                 reg_info: reg,
                 bounds: bounds.clone(),
                 value_index: bounds_map[&bounds],
-                index: registers.indexes.len()
+                index: registers.indexes.len(),
             };
-    
+
             registers.indexes.push(reg_obj.clone());
-            registers.regs.insert(reg_obj.reg_info.name.clone(), reg_obj);
+            registers
+                .regs
+                .insert(reg_obj.reg_info.name.clone(), reg_obj);
         }
 
         for alias in &reg_info.alias_info {
-            registers.aliases.insert(alias.role_str.to_owned(), alias.clone());
+            registers
+                .aliases
+                .insert(alias.role_str.to_owned(), alias.clone());
 
             if alias.role_str == "PC" {
                 registers.pc = Some(registers.get_register(&alias.reg).unwrap().clone());
             }
         }
-    
+
         registers
     }
 
@@ -164,34 +169,16 @@ impl Registers {
     #[inline]
     pub fn get_value(&self, index: usize) -> Value {
         let register = &self.indexes[index];
-
-        if register.reg_info.offset as i64 == -1 {
+        if register.reg_info.offset == -1i64 as u64 {
             return Value::Concrete(0, 0); // this is a zero register
         }
 
         let value = &self.values[register.value_index];
-        let size = register.reg_info.size;
-
-        if size == register.bounds.size {
-            return value.to_owned();
-        }
-
-        let offset: u64 = register.reg_info.offset - register.bounds.start;
-
-        match value {
-            Value::Concrete(val, t) => {
-                let mask: u64 = (1 << size) - 1;
-                Value::Concrete((val >> offset) & mask, *t)
-            },
-            Value::Symbolic(val, t) => {
-                if val.is_const() {
-                    let mask: u64 = (1 << size) - 1;
-                    Value::Concrete((val.as_u64().unwrap() >> offset) & mask, *t)
-                } else {
-                    Value::Symbolic(val.slice(
-                        (offset+size-1) as u32, offset as u32), *t)
-                }
-            }
+        if register.reg_info.size == register.bounds.size {
+            value.to_owned()
+        } else {
+            let offset = register.reg_info.offset - register.bounds.start;
+            value.slice(register.reg_info.size + offset - 1, offset)
         }
     }
 
@@ -200,7 +187,7 @@ impl Registers {
         //println!("reg {:?} {:?}", index, value);
         let register = &self.indexes[index];
 
-        if register.reg_info.offset as i64 == -1 {
+        if register.reg_info.offset == -1i64 as u64 {
             return; // this is a zero register
         }
 
@@ -210,7 +197,7 @@ impl Registers {
             self.values[register.value_index] = value;
         } else {
             let bound_size = register.bounds.size as u32;
-            let offset: u64 = register.reg_info.offset - register.bounds.start;
+            let offset = register.reg_info.offset - register.bounds.start;
             let old_value = &self.values[register.value_index];
 
             let mut new_sym;
@@ -221,26 +208,26 @@ impl Registers {
             // im pulling trig and just using new, the downside of not is too high
             match (value, old_value.to_owned()) {
                 (Value::Concrete(new, t1), Value::Concrete(old, _t2)) => {
-                    let new_mask: u64 = (1 << size) - 1; 
-                    let mask: u64 = !(new_mask << offset);
+                    let new_mask = (1 << size) - 1;
+                    let mask = !(new_mask << offset);
 
                     let new_value = (old & mask) + ((new & new_mask) << offset);
                     self.values[register.value_index] = Value::Concrete(new_value, t1);
                     return;
-                },
+                }
                 (Value::Concrete(new, t1), Value::Symbolic(old, _t2)) => {
                     new_sym = self.solver.bvv(new, size as u32);
-                    old_sym = old; 
+                    old_sym = old;
                     taint = t1; // | t2;
-                },
+                }
                 (Value::Symbolic(new, t1), Value::Concrete(old, _t2)) => {
                     old_sym = self.solver.bvv(old, bound_size);
                     new_sym = new;
                     taint = t1; // | t2;
-                },
+                }
                 (Value::Symbolic(new, t1), Value::Symbolic(old, _t2)) => {
-                    old_sym = old; 
-                    new_sym = new; 
+                    old_sym = old;
+                    new_sym = new;
                     taint = t1; // | t2;
                 }
             }
@@ -251,16 +238,16 @@ impl Registers {
             let mut new_value;
 
             if offset > 0 {
-                new_value = new_sym.concat(&old_sym.slice((offset-1) as u32, 0));
+                new_value = new_sym.concat(&old_sym.slice((offset - 1) as u32, 0));
                 let new_off = offset as u32 + size as u32;
                 if bound_size - new_off > 0 {
-                    new_value = old_sym.slice(bound_size-1, new_off).concat(&new_value);
+                    new_value = old_sym.slice(bound_size - 1, new_off).concat(&new_value);
                 }
             } else {
-                new_value = old_sym.slice(bound_size-1, size as u32).concat(&new_sym);
+                new_value = old_sym.slice(bound_size - 1, size as u32).concat(&new_sym);
             }
 
             self.values[register.value_index] = Value::Symbolic(new_value, taint);
         }
     }
-}
\ No newline at end of file
+}
