@@ -47,13 +47,23 @@ pub fn get_sims() -> Vec<Sim> {
         make_sim("putwchar", libc::putchar, 1),
         make_sim("getchar", libc::getchar, 0),
         make_sim("fgets", libc::fgets, 1),
+        make_sim("fgetc", libc::fgets, 1),
+        make_sim("fputs", libc::fputs, 2),
+        make_sim("fputc", libc::fputc, 2),
+        make_sim("feof", libc::feof, 1),
+        make_sim("fprintf", libc::fputs, 2), // fix
         make_sim("printf", libc::printf, 1), // fix
+        make_sim("perror", libc::perror, 1), 
         make_sim("fflush", libc::fflush, 0),
+        make_sim("scanf", libc::scanf, 1), // fix
         make_sim("__isoc99_scanf", libc::scanf, 1), // fix
         make_sim("fopen", libc::fopen, 2),
+        make_sim("freopen", libc::fopen, 3),
         make_sim("fread", libc::fread, 4),
         make_sim("fwrite", libc::fwrite, 4),
+        make_sim("ferror", libc::zero, 1),
         make_sim("fseek", libc::fseek, 2),
+        make_sim("ftell", libc::ftell, 1),
         make_sim("fclose", libc::fgets, 1),
         make_sim("strlen", libc::strlen, 1),
         make_sim("strnlen", libc::strnlen, 2),
@@ -61,6 +71,7 @@ pub fn get_sims() -> Vec<Sim> {
         make_sim("strcpy", libc::strcpy, 2),
         make_sim("stpcpy", libc::stpcpy, 2),
         make_sim("strncpy", libc::strncpy, 3),
+        make_sim("strxfrm", libc::strncpy, 3),
         make_sim("strcat", libc::strcat, 2),
         make_sim("strncat", libc::strncat, 3),
         make_sim("strdup", libc::strdup, 1),
@@ -72,6 +83,7 @@ pub fn get_sims() -> Vec<Sim> {
         make_sim("strrchr", libc::strrchr, 2),
         make_sim("strstr", libc::strstr, 2),
         make_sim("strcmp", libc::strcmp, 2),
+        make_sim("strcoll", libc::strcmp, 2),
         make_sim("strncmp", libc::strncmp, 3),
         make_sim("memmove", libc::memmove, 3),
         make_sim("memcpy", libc::memcpy, 3),
