@@ -69,18 +69,24 @@ fn simple() {
     }
 }
 
+// this takes a really long time now (3s)
+// cuz its doing the atoi also
 #[test]
 fn multi() {
     use crate::radius::{Radius, RadiusOption};
 
     let options = [RadiusOption::Debug(true)];
     let mut radius = Radius::new_with_options(Some("../tests/multi"), &options);
-    let _state = radius.entry_state();
-    // TODO fix this test
-    /*let mut new_state = radius.run_until(state, 0x11c2, &[0x11c9]).unwrap();
+    let mut state = radius.entry_state();
+
+    let arg1 = state.symbolic_value("arg1", 40);
+    let name = state.concrete_value(0, 0);
+
+    radius.set_argv_env(&mut state, &[name, arg1], &[]);
+    let mut new_state = radius.run_until(state, 0x11c2, &[0x11c9]).unwrap();
 
     let arg = new_state.registers.get_with_alias("A0");
-    println!("{:?} {:?}", arg, new_state.eval(&arg));*/
+    println!("arg: {}", new_state.eval(&arg).unwrap().as_u64().unwrap());
 }
 
 #[test]
